@@ -4,12 +4,16 @@ __all__ = ['Default', 'Apps']
 import re
 import typing as t
 
-from ...base.core import Foam
-from ...base.lib import lib
+from ...base.lib import tqdm
+
+if t.TYPE_CHECKING:
+    from ...base.core import Foam
 
 
 class Default:
-    def __init__(self, foam: Foam) -> None:
+    '''Default app'''
+
+    def __init__(self, foam: 'Foam') -> None:
         pass
 
     def __enter__(self) -> 'Default':
@@ -24,11 +28,14 @@ class Default:
     def close(self) -> None:
         pass
 
+
 class AppBase(Default):
-    def __init__(self, foam: Foam) -> None:
+    '''Base app'''
+
+    def __init__(self, foam: 'Foam') -> None:
         start = float(foam['foam']['system', 'controlDict', 'startTime'])
         end = float(foam['foam']['system', 'controlDict', 'endTime'])
-        self.pbar = lib['tqdm'].tqdm(total=end-start)
+        self.pbar = tqdm.tqdm(total=end-start)
         self._new = self._old = start
 
     def __exit__(self, type, value, traceback) -> None:
@@ -42,6 +49,7 @@ class AppBase(Default):
     def now(self, line: bytes) -> t.Optional[float]:
         raise NotImplementedError
 
+
 class AppByTimeI(AppBase):
     '''
     - solver:
@@ -117,33 +125,39 @@ class AppByTimeI(AppBase):
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/parallelProcessing/reconstructParMesh/reconstructParMesh.C
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/parallelProcessing/reconstructPar/reconstructPar.C
     '''
-    def now(self, line):
+
+    def now(self, line: bytes) -> t.Optional[float]:
         line = line.strip()
         if line.startswith(b'Time = '):
             return float(line[7:])
 
+
 class AppByTimeII(AppBase):
     '''
     - utility:
         - postProcessing:
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/postProcessing/dataConversion/foamToVTK/foamToVTK.C
     '''
-    def now(self, line):
+
+    def now(self, line: bytes) -> t.Optional[float]:
         line = line.strip()
         if line.startswith(b'Time: '):
             return float(line[6:])
 
+
 class AppByIterationI(AppBase):
     '''
     - solver:
         - electromagnetics:
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/solvers/electromagnetics/electrostaticFoam/electrostaticFoam.C
     '''
-    def now(self, line):
+
+    def now(self, line: bytes) -> t.Optional[float]:
         line = line.strip()
         if line.startswith(b'Iteration = '):
             return float(line[12:])
 
+
 class AppByIterationII(AppBase):
     '''
     - solver:
@@ -151,11 +165,13 @@ class AppByIterationII(AppBase):
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/solvers/stressAnalysis/solidEquilibriumDisplacementFoam/solidEquilibriumDisplacementFoam.C
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/solvers/stressAnalysis/solidDisplacementFoam/solidDisplacementFoam.C
     '''
-    def now(self, line):
+
+    def now(self, line: bytes) -> t.Optional[float]:
         line = line.strip()
         if line.startswith(b'Iteration: '):
             return float(line[11:])
 
+
 class AppByProcessor(AppBase):
     '''
     - utility:
@@ -163,17 +179,19 @@ class AppByProcessor(AppBase):
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/parallelProcessing/decomposePar/decomposePar.C
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/parallelProcessing/redistributePar/redistributePar.C
     '''
-    def __init__(self, foam: Foam):
+
+    def __init__(self, foam: 'Foam') -> None:
         start = 0
         end = foam.number_of_processors - 1
-        self.pbar = lib['tqdm'].tqdm(total=end-start)
+        self.pbar = tqdm.tqdm(total=end-start)
         self._new = self._old = start
 
-    def now(self, line):
+    def now(self, line: bytes) -> t.Optional[float]:
         line = line.strip()
         if line.startswith(b'Processor ') and line[10:].isdigit():
             return int(line[10:])
 
+
 class AppByOther(AppBase):
     '''
     - solver:
@@ -341,12 +359,13 @@ class AppByOther(AppBase):
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/surface/surfaceFind/surfaceFind.C
             - https://github.com/OpenFOAM/OpenFOAM-7/blob/master/applications/utilities/surface/surfaceMeshTriangulate/surfaceMeshTriangulate.C
     '''
-    def now(self, _):
+
+    def now(self, line: bytes) -> t.Optional[float]:
         raise NotImplementedError
 
 
 Apps = {}
-if lib['tqdm'] is not None:
+if tqdm.is_available():
     pattern = re.compile(r'\w+(?=\.C)')
     for App in [AppByTimeI, AppByTimeII, AppByIterationI, AppByIterationII, AppByProcessor]:
         for name in pattern.findall(App.__doc__):
