@@ -1,20 +1,26 @@
 __all__ = ['Foam']
 
 
+import copy
 import functools as f
 import gc
 import json
 import os
 import pathlib as p
 import typing as t
-import urllib
+import urllib.parse
+import urllib.request
 import warnings as w
 
-from .lib import lib
-from .parse import Parser
-from .type import Dict, List, Path, Data, Version
+from .lib import yaml
+from .type import Dict, FoamData, Path
+from ..parse import Parser
+from ..util.object.data import Data
+from ..util.object.version import Version
 
 if t.TYPE_CHECKING:
+    from typing_extensions import Self
+
     from ..app.command.core import Command
     from ..app.information.core import Information
     from ..app.postprocess.core import PostProcess
@@ -24,15 +30,15 @@ class Foam:
     '''Convert multiple dictionary type data to OpenFOAM test case
 
     Example:
-        >>> foam = Foam.from_demo('cavity')
+        >>> foam = Foam.from_remote_demo('cavity')
         >>> foam['foam']['system', 'controlDict', 'endTime'] = 1.0
         >>> foam.save('cavity')
         >>> foam.cmd.all_run()
     '''
 
-    __version__ = '0.11.8'
+    __version__ = Version.from_string('0.12.4')
 
-    def __init__(self, data: List, root: Path, warn: bool = True) -> None:
+    def __init__(self, data: FoamData, root: Path, warn: bool = True) -> None:
         self._list = data
         self._root = p.Path(root)
         self._dest: t.Optional[p.Path] = None
@@ -46,25 +52,27 @@ class Foam:
             openfoam = set(map(str, self.meta.get('openfoam', [])))
             if str(self.environ['WM_PROJECT_VERSION']) not in openfoam:
                 w.warn(f'OpenFOAM version mismatch: {root}')
-            version = self._parse(self.__version__)
-            current = self._parse(str(self.meta.get('version', '0.0.0')))
-            if (version.major, version.minor) < (current.major, current.minor):
+            version = Version.from_string(str(self.meta.get('version', '0.0.0')))
+            if self.__version__ < version:
                 w.warn('Forward compatibility is not yet guaranteed')
-            elif (version.major, version.minor) > (current.major, current.minor):
+            elif self.__version__ > version:
                 w.warn('Backward compatibility is not yet guaranteed')
 
-    def __getitem__(self, key: str) -> t.Optional['Data']:
+    def __getitem__(self, key: str) -> t.Optional[Data]:
         try:
-            return Data(self._list[self.meta['order'].index(key)])
+            return Data.from_any(self._list[self.meta['order'].index(key)])
         except ValueError:
             return None
 
     def __repr__(self) -> str:
+        return f'Foam({self._list!r}, {self._root!r})'
+
+    def __str__(self) -> str:
         return f'<Foam @ "{self._root.absolute().as_posix()}">'
 
     @classmethod
     def list_demos(cls) -> t.List[str]:
-        root = p.Path(__file__).parents[1] / 'demo' / os.environ['WM_PROJECT_VERSION']
+        root = p.Path(__file__).parents[1] / 'static' / 'demo' / os.environ['WM_PROJECT_VERSION']
         return sorted(
             path.stem
             for path in root.iterdir()
@@ -72,31 +80,50 @@ class Foam:
         )
 
     @classmethod
-    def from_demo(cls, name: str = 'cavity') -> t.Self:
+    def from_demo(cls, name: str = 'cavity', warn: bool = False) -> 'Self':
         version = os.environ['WM_PROJECT_VERSION']
         name = name if name.endswith('.yaml') else f'{name}.yaml'
-        path = p.Path(__file__).parents[1] / 'demo' / version / name
+        path = p.Path(__file__).parents[1] / 'static' / 'demo' / version / name
         try:
-            self = cls.from_file(path, warn=False)
+            self = cls.from_path(path, warn=warn)
         except FileNotFoundError:
             raise FileNotFoundError(f'No such demo: "{name[:-5]}" not in {cls.list_demos()}')
         else:
-            print(f'Foam.from_file(\'{path.as_posix()}\', warn=False)')
+            print(f'Foam.from_path(\'{path.as_posix()}\', warn={warn})')
             self.meta.setdefault('openfoam', []).append(version)
-            self.meta['version'] = cls.__version__
+            self.meta['version'] = cls.__version__.to_string()
             return self
 
     @classmethod
-    def from_demos(cls) -> t.List[t.Self]:
-        return list(map(cls.from_demo, cls.list_demos()))
+    def from_demos(cls, warn: bool = False) -> t.List['Self']:
+        return list(map(lambda name: cls.from_demo(name, warn), cls.list_demos()))
 
     @classmethod
-    def from_remote_file(cls, url: str, **kwargs: t.Any) -> t.Self:
-        with urllib.request.urlopen(url) as f:
+    def from_remote_demo(cls, name: str = 'cavity', timeout: t.Optional[float] = None, warn: bool = False) -> 'Self':
+        version = os.environ['WM_PROJECT_VERSION']
+        name = name if name.endswith('.yaml') else f'{name}.yaml'
+        url = f'https://raw.githubusercontent.com/iydon/of.yaml/main/foam/static/demo/{version}/{name}'
+        try:
+            self = cls.from_remote_path(url, timeout, warn=warn)
+        except:  # urllib.error.URLError
+            raise FileNotFoundError(f'No such demo: "{url}"')
+        else:
+            print(f'Foam.from_remote_path(\'{url}\', warn={warn})')
+            self.meta.setdefault('openfoam', []).append(version)
+            self.meta['version'] = cls.__version__.to_string()
+            return self
+
+    @classmethod
+    def from_remote_demos(cls, timeout: t.Optional[float] = None, warn: bool = False) -> t.List['Self']:
+        return list(map(lambda name: cls.from_remote_demo(name, timeout, warn), cls.list_demos()))
+
+    @classmethod
+    def from_remote_path(cls, url: str, timeout: t.Optional[float] = None, warn: bool = True) -> 'Self':
+        with urllib.request.urlopen(url, timeout=timeout) as f:
             text = f.read()
         split_url = urllib.parse.urlsplit(url)
         path = p.Path(split_url.path)
-        self = cls.from_text(text, '.', path.suffix, **kwargs)
+        self = cls.from_text(text, '.', path.suffix, warn=warn)
         self.parser.url.set_split_url(split_url)
         for old in self['static'] or []:
             types = tuple(old.get('type', []))
@@ -104,13 +131,24 @@ class Foam:
         return self
 
     @classmethod
-    def from_file(cls, path: Path, **kwargs: t.Any) -> t.Self:
-        '''Supported format: json, yaml'''
+    def from_path(cls, path: Path, warn: bool = True) -> 'Self':
+        '''Supported path mode: file, directory'''
         path = p.Path(path)
-        return cls.from_text_via_suffix(path.read_text(), path.parent, path.suffix, **kwargs)
+        if path.is_file():
+            return cls.from_text(path.read_text(), path.parent, path.suffix, warn=warn)
+        elif path.is_dir():
+            return cls.from_openfoam(path)
+        else:
+            raise Exception(f'Mode "{path.stat().st_mode}" is not supported')
 
     @classmethod
-    def from_text(cls, text: str, root: Path, suffix: t.Optional[str] = None, **kwargs: t.Any) -> t.Self:
+    def from_openfoam(cls, path: Path, **kwargs: t.Any) -> 'Self':
+        '''From OpenFOAM directory'''
+        return Parser.to_foam(path, **kwargs)
+
+    @classmethod
+    def from_text(cls, text: str, root: Path, suffix: t.Optional[str] = None, warn: bool = True) -> 'Self':
+        '''Supported format: json, yaml'''
         mapper = [
             ({'.json'}, cls.from_json),
             ({'.yaml', '.yml'}, cls.from_yaml),
@@ -118,29 +156,32 @@ class Foam:
         if suffix is not None:
             for suffixes, method in mapper:
                 if suffix in suffixes:
-                    return method(text, root, **kwargs)
+                    return method(text, root, warn=warn)
         else:
             for _, method in mapper:
                 try:
-                    return method(text, root, **kwargs)
+                    return method(text, root, warn=warn)
                 except Exception as e:
                     print(method.__name__, e)
         raise Exception(f'Suffix "{suffix}" is not supported or not recognized')
 
     @classmethod
-    def from_json(cls, text: str, root: Path, **kwargs: t.Any) -> t.Self:
+    def from_json(cls, text: str, root: Path, warn: bool = True) -> 'Self':
         data = json.loads(text)
-        return cls(data, root, **kwargs)
+        return cls(data, root, warn=warn)
 
     @classmethod
-    def from_yaml(cls, text: str, root: Path, **kwargs: t.Any) -> t.Self:
-        data = list(lib['yaml'].load_all(text, Loader=lib['SafeLoader']))
-        return cls(data, root, **kwargs)
+    def from_yaml(cls, text: str, root: Path, warn: bool = True) -> 'Self':
+        return cls(list(yaml.load_all(text)), root, warn=warn)
 
     @classmethod
-    def as_placeholder(cls) -> t.Self:
+    def as_placeholder(cls) -> 'Self':
         return cls([{}], '', warn=False)
 
+    @property
+    def data(self) -> Data:
+        return Data.from_list(self._list)
+
     @property
     def meta(self) -> Dict:
         '''Meta information'''
@@ -180,6 +221,16 @@ class Foam:
             self._post = PostProcess.from_foam(self)
         return self._post
 
+    @property
+    def destination(self) -> p.Path:
+        assert self._dest is not None, 'Please call `Foam::save` method first'
+
+        return self._dest
+
+    @destination.setter
+    def destination(self, dest: Path) -> None:
+        self._dest = p.Path(dest)
+
     @f.cached_property
     def application(self) -> str:
         '''Inspired by `getApplication`
@@ -205,7 +256,7 @@ class Foam:
             return 1
 
     @f.cached_property
-    def pipeline(self) -> t.List[t.Union[str, t.Dict[str, t.Any]]]:
+    def pipeline(self) -> t.List[t.Union[str, Dict]]:
         return (self['other'] or {}).get('pipeline', [])
 
     @f.cached_property
@@ -240,7 +291,12 @@ class Foam:
             count = min(count, len(grids)-grids.count('1'))
         return count
 
-    def save(self, dest: Path, paraview: bool = True) -> t.Self:
+    def copy(self) -> 'Self':
+        other = self.__class__(copy.deepcopy(self._list), self._root)
+        other._dest = self._dest
+        return other
+
+    def save(self, dest: Path, paraview: bool = True) -> 'Self':
         '''Persist case to hard disk'''
         self._dest = p.Path(dest)
         self._dest.mkdir(parents=True, exist_ok=True)
@@ -250,7 +306,7 @@ class Foam:
             self._write(self._dest/'paraview.foam', '')
         return self
 
-    def reset(self) -> t.Self:
+    def reset(self) -> 'Self':
         self._dest = self._cmd = self._info = self._post = None
         for obj in gc.get_objects():
             if isinstance(obj, f._lru_cache_wrapper):
@@ -297,7 +353,4 @@ class Foam:
     def _path(self, *parts: str) -> p.Path:
         # TODO: use "prefix" will cause some of the `Command` methods to fail
         prefix = '.'  # (self['other'] or {}).get('directory', '.')
-        return self._dest / prefix / p.Path(*parts)
-
-    def _parse(self, version: str) -> Version:
-        return Version(*map(int, version.split('.')))
+        return self.destination / prefix / p.Path(*parts)
